texture tex0 < string name = "River.tga"; >;
texture tex1 < string name = "RiverMovement.tga"; >;
texture tex2 < string name = "TerraIncog.dds"; >;		// Borders texture
texture tex3 < string name = "colormap_water.dds"; >;


float4x4 WorldMatrix; 
float4x4 ViewMatrix; 
float4x4 ProjectionMatrix; 
float4x4 AbsoluteWorldMatrix;
float	 vTime;

sampler BaseTexture  =
sampler_state
{
    Texture = <tex0>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
    AddressU = Wrap;
    AddressV = Wrap;
};

sampler AnimatedTexture  =
sampler_state
{
    Texture = <tex1>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
    AddressU = Wrap;
    AddressV = Wrap;
};

sampler TerraIncognitaTexture =
sampler_state
{
    Texture = <tex2>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};

sampler Water =
sampler_state
{
    Texture = <tex3>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};


struct VS_INPUT
{
    float4 vPosition  : POSITION;
    float2 vTexCoord0 : TEXCOORD0;
    float2 vTexCoord1 : TEXCOORD1;
};

struct VS_OUTPUT
{
    float4  vPosition  : POSITION;
    float2  vTexCoord0 : TEXCOORD0;
    float2  vTexCoord1 : TEXCOORD1;
    float2  vTerrainTexCoord : TEXCOORD2;
    float2  vWaterTexCoord : TEXCOORD3;
};


const float2 off = float2(0.25/1024, 0.25/512);
const float2 offx = float2(0.75/1024, 0.0);
const float2 offy = float2(0.0, 0.75/512);

VS_OUTPUT VertexShader_River(const VS_INPUT v )
{
	VS_OUTPUT Out = (VS_OUTPUT)0;
	float4x4 WorldView = mul(WorldMatrix, ViewMatrix);
	float4 Q = mul(v.vPosition, WorldMatrix);
	Q.y = 0.31;
	float3 P = mul(Q, (float4x3)ViewMatrix);
	//float3 P = mul(v.vPosition, (float4x3)WorldView);
	Out.vPosition  = mul(float4(P, 1), ProjectionMatrix);

	Out.vTexCoord0 = v.vTexCoord0;
	Out.vTexCoord1 = v.vTexCoord1 * float2(1.0,0.333); 

	float4 WorldPosition = mul( v.vPosition, AbsoluteWorldMatrix );
	Out.vTerrainTexCoord  = float2( (WorldPosition.x+0.5)/2048.0f, (WorldPosition.z+0.5)/1024.0f );

	Out.vWaterTexCoord.x = saturate(3*mul( float3(0,1,0) , (float4x3)WorldView).y -0.35); //zoom
	Out.vWaterTexCoord.y = Q.y;

	return Out;
}

float4 PixelShader_River( VS_OUTPUT v ) : COLOR
{
	float4 OutColor = tex2D( BaseTexture, v.vTexCoord0.xy );
	float4 Movement = tex2D( AnimatedTexture, v.vTexCoord1  );

	//OutColor.rgb = float3(0.42,0.42,0.65);
//	OutColor.rgb *= Movement.rgb;
	//OutColor.rgb *= saturate ( OutColor.a * 10 - 8);
	OutColor *= 0.92; //was only for .a
	

	OutColor.rgb = lerp(OutColor.rgb, float3(0.3,0.3, 0.55), 0.75*(1-v.vWaterTexCoord.x));
  // float3(0.3,0.3, 0.55) -- was zoomed out colour above
	// Terra incognita
	
	float TerraIncognita = tex2D( TerraIncognitaTexture, v.vTerrainTexCoord ).g;
	if (TerraIncognita < 0.26) OutColor.a *= 0.5+ 2 *TerraIncognita;
//	 else  OutColor.a *= 1-saturate(5*4 *(TerraIncognita-0.25)/3);
	 
  float local = saturate(tex2D( TerraIncognitaTexture, v.vTerrainTexCoord + offx).g -0.25)+
  	saturate(tex2D( TerraIncognitaTexture, v.vTerrainTexCoord - offx).g -0.25)+
  	saturate(tex2D( TerraIncognitaTexture, v.vTerrainTexCoord + offy).g -0.25)+
  	saturate(tex2D( TerraIncognitaTexture, v.vTerrainTexCoord - offy).g -0.25)+
  	saturate(TerraIncognita-0.25);
	OutColor = lerp(OutColor, 0.0, saturate(local*3));
	 
	
	//if (v.vWaterTexCoord.y < 0.45) OutColor.a = 0;
  OutColor.a *= 1-3*tex2D( Water, v.vTerrainTexCoord ).b;
	return OutColor;
}

technique RiverShader
{
	pass p0
	{
		ALPHABLENDENABLE = True;
		ALPHATESTENABLE = True;

		VertexShader = compile vs_1_1 VertexShader_River();
		PixelShader = compile ps_2_0 PixelShader_River();
	}
}
